import React, { useState, useEffect, useCallback } from 'react'
import { Input, message, Popover } from 'antd'
import { searchAdress } from '../../../../apis/process'
import { debounce } from '../../../../utils/index'

const SearchLocation = (props) => {

  const areaName = window?.globalConfig?.mapsettings?.areasettings?.areaName || ''
  const { value, onChange, schema, addons } = props
  const { disabled, placeholder, presetValue } = schema
  const [options, setOptions] = useState([])

  const getAdress = async (value) => {
    const { code, data } = await searchAdress({ keywords: `${areaName}${value}`, region: areaName })
    if (code === '0') {
      setOptions(data?.pois)
    } else {
      message.error('请求错误!')
    }
  }

  const doSomeThing = useCallback(debounce(getAdress), [])

  const onSearch = async (e) => {
    doSomeThing(e.target.value)
    onChange(e.target.value)
  }

  useEffect(() => {
    if (addons) {
      addons.setValue(addons.dataPath, presetValue || '')
    }
  }, [presetValue])

  const content = (
    <div>
      {
        (options || []).map(v => <p key={v.name} onClick={() => onChange(v.name)}><a>{v.name}</a></p>)
      }
    </div>
  )

  return (
    <div>
      <Popover content={content} title="地名选择" trigger="hover" placement="topLeft">
        <Input
          disabled={disabled}
          placeholder={placeholder}
          value={value}
          onChange={onSearch}
        />
      </Popover>
    </div>
  )
}

export default SearchLocation
